﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;

namespace FunctionGen
{
    public partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();
            string[] ports = SerialPort.GetPortNames();
            foreach (string PortName in ports)
            {
                comboBoxPortNames.Items.Add(PortName);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set initial conditions for channel 1
            numericUpDownFreqChannel1.Value = 1000; // Start with 1KHz
            comboBoxFreqTypeChannel1.SelectedIndex = 1; //Select Hz
            comboBoxWaveformChannel1.SelectedIndex = 0; // Select sinewave
            numericUpDownAmplitudeChannel1.Value = 5; // Set output to full
            numericUpDownPhaseChannel1.Value = 0;
            numericUpDownDutyCycleChannel1.Value = 50;

            // Set initial conditions for channel 2
            numericUpDownFreqChannel2.Value = 1000; // Start with 1KHz
            comboBoxFreqTypeChannel2.SelectedIndex = 1; //Select Hz
            comboBoxWaveformChannel2.SelectedIndex = 0; // Select sinewave
            numericUpDownAmplitudeChannel2.Value = 5; // Set output to full
            numericUpDownPhaseChannel2.Value = 0;
            numericUpDownDutyCycleChannel2.Value = 50;

            // Set initial conditions for Sweep Generator
            comboBoxSweepStartMultiplier.SelectedIndex = 1;
            comboBoxSweepEndMultiplier.SelectedIndex = 1;
            comboBoxSweepLinLog.SelectedIndex = 0;

        }

        private void buttonConnectToDDS_Click(object sender, EventArgs e)
        {

            serialPort1.PortName = comboBoxPortNames.Items[comboBoxPortNames.SelectedIndex].ToString();
            try
            {
                serialPort1.Open();
                serialPort1.ReadTimeout = 1000; // Timeout when reading if nothing ready
             }
            catch
            {
                MessageBox.Show("Unable to open port. Is it the correct one?", "Error");
            }

            try
            {
                serialPort1.WriteLine(":"); // Clear command buffer
                serialPort1.WriteLine(":r1c");
                string s1 = serialPort1.ReadLine().Substring(4);
                s1 = s1.Substring(0, s1.Length - 1);
                serialPort1.WriteLine(":r0c");
                string s2 = serialPort1.ReadLine().Substring(4);
                s2 = s2.Substring(0, s2.Length - 1);
                toolStripStatusLabel1.Text = "Connected to DDS Model " + s1 + " Version " + s2;
            }
            catch (Exception)
            {
                MessageBox.Show("There was a problem initiating communications.", "Error");
            }

            buttonDisconnectDDS.Enabled = true;
            buttonConnectToDDS.Enabled = false;
            panelChannel1.Enabled = true;
            panelChannel2.Enabled = true;
            panelSweepGenerator.Enabled = true;
            
       }

        private void buttonDisconnectDDS_Click(object sender, EventArgs e)
        {
            panelChannel1.Enabled = false;
            panelChannel2.Enabled = false;
            panelSweepGenerator.Enabled = false;
            serialPort1.Close();
            buttonDisconnectDDS.Enabled = false;
            buttonConnectToDDS.Enabled = true;
            toolStripStatusLabel1.Text = "Not connected to DDS ";
        }

        private void buttonSetValuesChannel1_Click(object sender, EventArgs e)
        {
            int iFreq = ( int ) numericUpDownFreqChannel1.Value;
            switch (comboBoxFreqTypeChannel1.SelectedIndex)
            {
                case 1: iFreq *= 100;
                    break;
                case 2: iFreq *= 100000;
                    break;
            }
            int iWaveform = (int)comboBoxWaveformChannel1.SelectedIndex;
            int iAmp = (int)numericUpDownAmplitudeChannel1.Value * 100;
            int iPhase = (int) numericUpDownPhaseChannel1.Value;
            int iDuty = (int) numericUpDownDutyCycleChannel1.Value * 10;
            int iOffset = (int)numericUpDownOffsetChannel1.Value + 120;
            ProgramDDS(1, iWaveform, iFreq, iAmp, iPhase, iOffset, iDuty);
        }

        private void buttonSetValuesChannel2_Click(object sender, EventArgs e)
        {
            int iFreq = ( int ) numericUpDownFreqChannel2.Value;
            switch (comboBoxFreqTypeChannel2.SelectedIndex)
            {
                case 1: iFreq *= 100;
                    break;
                case 2: iFreq *= 100000;
                    break;
            }
            int iWaveform = (int)comboBoxWaveformChannel2.SelectedIndex;
            
            int iAmp = (int)numericUpDownAmplitudeChannel2.Value * 100;
            int iPhase = (int)numericUpDownPhaseChannel2.Value;
            int iDuty = (int)numericUpDownDutyCycleChannel2.Value * 10;
            int iOffset = (int)numericUpDownOffsetChannel2.Value + 120;
            ProgramDDS(2, iWaveform, iFreq, iAmp, iPhase, iOffset, iDuty);
        }

        private void ProgramDDS(int Channel, int Waveform, int Freq, int Amp, int Phase, int Offset, int Duty)
        {
            string ch = ":s" + Channel.ToString();
            
            // Set Frequency
            string sFreq = ch + "f" + Freq.ToString();
            string sAmp = ch + "a" + Amp.ToString();
            string sPhase = ch + "p" + Phase.ToString();
            string sOffset = ch + "o" + Offset.ToString();
            string sDuty = ch + "d" + Duty.ToString();
            string sWave = ch + "w" + Waveform.ToString();
            SendCommand(sFreq);
            SendCommand(sAmp);
            SendCommand(sPhase);
            SendCommand(sOffset);
            SendCommand(sDuty);
            SendCommand(sWave);

       }

        // This method handles sending the commands to the Function Generator
        private void SendCommand( string command )
        {
            string s = "";
            try
            {
                serialPort1.WriteLine( command );
                Thread.Sleep(100); // Give the unit time to respond to the command and reply
                s = serialPort1.ReadLine();
            }
            catch (Exception)
            {              
                MessageBox.Show("Error writing");
            }
        }

        private void buttonOutputOnChannel1_Click(object sender, EventArgs e)
        {
            SendCommand(":s1b1");
        }

        private void buttonOutputOffChannel1_Click(object sender, EventArgs e)
        {
            SendCommand(":s1b0");
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
                serialPort1.Close();
            Application.Exit();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("DDS Function Generator Controller V0.02\r\nBy Chris Mower\r\nCreative Commons copyright 2015");
        }

        private void buttonStartSweep_Click(object sender, EventArgs e)
        {
            buttonStartSweep.Enabled = false;
            buttonStopSweep.Enabled = true;

            int iFreq = (int)numericUpDownSweepStartFreq.Value;
            switch (comboBoxSweepStartMultiplier.SelectedIndex)
            {
                case 1: iFreq *= 100;
                    break;
                case 2: iFreq *= 100000;
                    break;
            }
            string sStart = ":s3f" + iFreq.ToString();

            iFreq = (int)numericUpDownSweepEndFreq.Value;
            switch (comboBoxSweepEndMultiplier.SelectedIndex)
            {
                case 1: iFreq *= 100;
                    break;
                case 2: iFreq *= 100000;
                    break;
            }
            string sStop = ":s4f" + iFreq.ToString();

            string sTime = ":s5t" + numericUpDownSweepTime.Value.ToString();
            string sMode = ":s7b" + comboBoxSweepLinLog.SelectedIndex.ToString();

            SendCommand(sStart);
            SendCommand(sStop);
            SendCommand(sTime);
            SendCommand(sMode);
            SendCommand(":s8b1"); // Now start the sweep
        }

        private void buttonStopSweep_Click(object sender, EventArgs e)
        {
            SendCommand(":s8b0"); // Send the stop command
            buttonStopSweep.Enabled = false;
            buttonStartSweep.Enabled = true;
        }
    }
}
